class _StringTable:
    # CSV Header Strings
    ClientId = "Client Id"
    Type = "Type"
    Status = "Status"
    Campaign = "Campaign"
    Id = "Id"
    BusinessId = "Business Id"
    ParentId = "Parent Id"
    TimeZone = "Time Zone"
    Budget = "Budget"
    BudgetType = "Budget Type"
    BudgetName = "Budget Name"
    BudgetId = "Budget Id"
    KeywordVariantMatchEnabled = "KeywordVariantMatchEnabled"
    AdGroup = "Ad Group"
    Keyword = "Keyword"
    TextAd = "Text Ad"
    ProductAd = "Product Ad"
    AppInstallAd = "App Install Ad"
    DynamicSearchAd = "Dynamic Search Ad"
    ExpandedTextAd = "Expanded Text Ad"
    Title = "Title"
    EditorialStatus = "Editorial Status"
    EditorialAppealStatus = "Editorial Appeal Status"
    Error = "Error"
    ErrorNumber = "Error Number"
    EditorialLocation = "Editorial Location"
    EditorialTerm = "Editorial Term"
    EditorialReasonCode = "Editorial Reason Code"
    MigrationErrors = "Migration Errors"
    DisplayUrl = "Display Url"
    DestinationUrl = "Destination Url"
    BusinessName = "Business Name"
    PhoneNumber = "Phone Number"
    PromotionalText = "Promotion"
    MatchType = "Match Type"
    Param1 = "Param1"
    Param2 = "Param2"
    Param3 = "Param3"
    DevicePreference = "Device Preference"
    CampaignNegativeKeyword = "Campaign Negative Keyword"
    CampaignNegativeSite = "Campaign Negative Site"
    AdGroupNegativeKeyword = "Ad Group Negative Keyword"
    KeywordNegativeKeyword = "Keyword Negative Keyword"
    AdGroupNegativeSite = "Ad Group Negative Site"
    Text = "Text"
    Website = "Website"
    Target = "Target"
    PhysicalIntent = "Physical Intent"
    Bid = "Bid"
    BidAdjustment = "Bid Adjustment"
    SubType = "Sub Type"
    OsNames = "OS Names"
    StartDate = "Start Date"
    EndDate = "End Date"
    NetworkDistribution = "Network Distribution"
    SearchNetwork = "Search Network"
    ContentNetwork = "Content Network"
    PricingModel = "Pricing Model"
    Language = "Language"
    SearchBid = "Search Bid"
    ContentBid = "Content Bid"
    AdRotation = "Ad Rotation"
    Account = "Account"
    SyncTime = "Sync Time"
    Name = "Name"
    LastModifiedTime = "Modified Time"
    AdFormatPreference = "Ad Format Preference"

    # Entity Types
    SemanticVersion = "Format Version"
    LocationTarget = "Location Target"
    RadiusTarget = "Radius Target"
    BusinessTarget = "Business Location Target"
    RadiusTargetId = "Radius Target Id"
    NegativeLocationTarget = "Negative Location Target"
    AgeTarget = "Age Target"
    GenderTarget = "Gender Target"
    DeviceOsTarget = "DeviceOS Target"
    Radius = "Radius"
    Unit = "Unit"
    TargetAll = "Target All"
    SiteLinksAdExtension = "Sitelink Ad Extension"
    LocationAdExtension = "Location Ad Extension"
    CallAdExtension = "Call Ad Extension"
    ImageAdExtension = "Image Ad Extension"
    SocialNetworkAdExtension = "Social Network Ad Extension"
    Version = "Version"

    # Image Ad Extension
    AltText = "Alternative Text"
    MediaIds = "Media Ids"
    AccountImageAdExtension = "Account Image Ad Extension"
    CampaignImageAdExtension = "Campaign Image Ad Extension"
    AdGroupImageAdExtension = "Ad Group Image Ad Extension"

    # SiteLink Ad Extensions
    SiteLinkExtensionOrder = "Sitelink Extension Order"
    SiteLinkDisplayText = "Sitelink Extension Link Text"
    SiteLinkDestinationUrl = "Sitelink Extension Destination Url"
    SiteLinkDescription1 = "Sitelink Extension Description1"
    SiteLinkDescription2 = "Sitelink Extension Description2"
    CampaignSiteLinksAdExtension = "Campaign Sitelink Ad Extension"
    AdGroupSiteLinksAdExtension = "AdGroup Sitelink Ad Extension"

    # Location Ad Extensions
    AddressLine1 = "Address Line 1"
    AddressLine2 = "Address Line 2"
    PostalCode = "Postal Code"
    City = "City"
    StateOrProvince = "State Or Province Code"
    ProvinceName = "Province Name"
    GeoCodeStatus = "Geo Code Status"
    IconMediaId = "Map Icon"
    ImageMediaId = "Business Icon"
    AccountLocationAdExtension = "Account Location Ad Extension"
    CampaignLocationAdExtension = "Campaign Location Ad Extension"

    # Call Ad Extensions
    CountryCode = "Country Code"
    IsCallOnly = "Call Only"
    CampaignCallAdExtension = "Campaign Call Ad Extension"
    IsCallTrackingEnabled = "Call Tracking Enabled"
    RequireTollFreeTrackingNumber = "Toll Free"

    # Structured Snippet Ad Extension
    StructuredSnippetAdExtension = "Structured Snippet Ad Extension"
    AccountStructuredSnippetAdExtension = "Account Structured Snippet Ad Extension"
    CampaignStructuredSnippetAdExtension = "Campaign Structured Snippet Ad Extension"
    AdGroupStructuredSnippetAdExtension = "Ad Group Structured Snippet Ad Extension"
    StructuredSnippetHeader = "Structured Snippet Header"
    StructuredSnippetValues = "Structured Snippet Values"

    # Sitelink2 Ad Extension
    Sitelink2AdExtension = "Sitelink2 Ad Extension"
    AdGroupSitelink2AdExtension = "Ad Group Sitelink2 Ad Extension"
    CampaignSitelink2AdExtension = "Campaign Sitelink2 Ad Extension"
    AccountSitelink2AdExtension = "Account Sitelink2 Ad Extension"

    # Editorial Rejection Reasons
    PublisherCountries = "Publisher Countries"

    # BTE Types
    KeywordFirstPageBidType = "Keyword First Page Bid"
    KeywordMainLineBidType = "Keyword Main Line Bid"
    KeywordBestPositionBidType = "Keyword Best Position Bid"

    ProductCondition1 = "Product Condition 1"
    ProductCondition2 = "Product Condition 2"
    ProductCondition3 = "Product Condition 3"
    ProductCondition4 = "Product Condition 4"
    ProductCondition5 = "Product Condition 5"
    ProductCondition6 = "Product Condition 6"
    ProductCondition7 = "Product Condition 7"
    ProductCondition8 = "Product Condition 8"
    ProductValue1 = "Product Value 1"
    ProductValue2 = "Product Value 2"
    ProductValue3 = "Product Value 3"
    ProductValue4 = "Product Value 4"
    ProductValue5 = "Product Value 5"
    ProductValue6 = "Product Value 6"
    ProductValue7 = "Product Value 7"
    ProductValue8 = "Product Value 8"

    BingMerchantCenterId = "Store Id"
    BingMerchantCenterName = "Store Name"

    # App Ad Extension
    AppAdExtension = "App Ad Extension"
    AppPlatform = "App Platform"
    AppStoreId = "App Id"
    AccountAppAdExtension = "Account App Ad Extension"
    CampaignAppAdExtension = "Campaign App Ad Extension"
    AdGroupAppAdExtension = "Ad Group App Ad Extension"
    IsTrackingEnabled = "Tracking Enabled"

    # Misc Stuff
    EntityActiveStatus = "Active"
    EntityDeletedStatus = "Deleted"
    IntraFieldSeparator = ';'
    Active = "Active"
    Latitude = "Latitude"
    Longitude = "Longitude"

    # BI
    Spend = "Spend"
    Impressions = "Impressions"
    Clicks = "Clicks"
    CTR = "CTR"
    AvgCPC = "Avg CPC"
    AvgCPM = "Avg CPM"
    AvgPosition = "Avg position"
    Conversions = "Conversions"
    CPA = "CPA"
    QualityScore = "Quality Score"
    KeywordRelevance = "Keyword Relevance"
    LandingPageRelevance = "Landing Page Relevance"
    LandingPageUserExperience = "Landing Page User Experience"

    # DayTime Criterion
    FromHour = "From Hour"
    ToHour = "To Hour"
    FromMinute = "From Minute"
    ToMinute = "To Minute"

    # Shared Entities
    NegativeKeywordList = "Negative Keyword List"
    CampaignNegativeKeywordList = "Campaign Negative Keyword List Association"
    ListNegativeKeyword = "Shared Negative Keyword"

    # Subtypes
    MetroAreaSubType = "Metro Area"
    CountrySubType = "Country"
    StateSubType = "State"
    CitySubType = "City"
    PostalCodeSubType = "Postal Code"

    # Bing Shopping Campaigns
    IsExcluded = "Is Excluded"
    ParentAdGroupCriterionId = "Parent Criterion Id"
    CampaignType = "Campaign Type"
    CampaignPriority = "Priority"
    LocalInventoryAdsEnabled = "LocalInventoryAdsEnabled"

    # V10 added
    FieldPath = "Field Path"

    # Upgrade URL
    FinalUrl = "Final Url"
    FinalMobileUrl = "Mobile Final Url"
    TrackingTemplate = "Tracking Template"
    CustomParameter = "Custom Parameter"

    # Review Ad Extension
    ReviewAdExtension = "Review Ad Extension"
    AccountReviewAdExtension = "Account Review Ad Extension"
    CampaignReviewAdExtension = "Campaign Review Ad Extension"
    AdGroupReviewAdExtension = "Ad Group Review Ad Extension"
    IsExact = "Is Exact"
    Source = "Source"
    Url = "Url"

    # Price Ad Extension
    PriceAdExtension = "Price Ad Extension"
    AccountPriceAdExtension = "Account Price Ad Extension"
    CampaignPriceAdExtension = "Campaign Price Ad Extension"
    AdGroupPriceAdExtension = "Ad Group Price Ad Extension"
    PriceExtensionType = "Price Extension Type"
    Header1 = "Header 1"
    Header2 = "Header 2"  
    Header3 = "Header 3"
    Header4 = "Header 4"
    Header5 = "Header 5"
    Header6 = "Header 6"
    Header7 = "Header 7"
    Header8 = "Header 8"
    PriceDescription1 = "Price Description 1"
    PriceDescription2 = "Price Description 2"
    PriceDescription3 = "Price Description 3"
    PriceDescription4 = "Price Description 4"
    PriceDescription5 = "Price Description 5"
    PriceDescription6 = "Price Description 6"
    PriceDescription7 = "Price Description 7"
    PriceDescription8 = "Price Description 8"
    FinalUrl1 = "Final Url 1"
    FinalUrl2 = "Final Url 2"
    FinalUrl3 = "Final Url 3"
    FinalUrl4 = "Final Url 4"
    FinalUrl5 = "Final Url 5"
    FinalUrl6 = "Final Url 6"
    FinalUrl7 = "Final Url 7"
    FinalUrl8 = "Final Url 8"
    FinalMobileUrl1 = "Final Mobile Url 1"
    FinalMobileUrl2 = "Final Mobile Url 2"
    FinalMobileUrl3 = "Final Mobile Url 3"
    FinalMobileUrl4 = "Final Mobile Url 4"
    FinalMobileUrl5 = "Final Mobile Url 5"
    FinalMobileUrl6 = "Final Mobile Url 6"
    FinalMobileUrl7 = "Final Mobile Url 7"
    FinalMobileUrl8 = "Final Mobile Url 8"
    Price1 = "Price 1" 
    Price2 = "Price 2"
    Price3 = "Price 3"
    Price4 = "Price 4"
    Price5 = "Price 5"
    Price6 = "Price 6"
    Price7 = "Price 7"
    Price8 = "Price 8"
    CurrencyCode1 = "Currency Code 1"
    CurrencyCode2 = "Currency Code 2"
    CurrencyCode3 = "Currency Code 3"
    CurrencyCode4 = "Currency Code 4"
    CurrencyCode5 = "Currency Code 5"
    CurrencyCode6 = "Currency Code 6"
    CurrencyCode7 = "Currency Code 7"
    CurrencyCode8 = "Currency Code 8"
    PriceUnit1 = "Price Unit 1"
    PriceUnit2 = "Price Unit 2"
    PriceUnit3 = "Price Unit 3"
    PriceUnit4 = "Price Unit 4"
    PriceUnit5 = "Price Unit 5"
    PriceUnit6 = "Price Unit 6"
    PriceUnit7 = "Price Unit 7"
    PriceUnit8 = "Price Unit 8"
    PriceQualifier1 = "Price Qualifier 1"
    PriceQualifier2 = "Price Qualifier 2"
    PriceQualifier3 = "Price Qualifier 3"
    PriceQualifier4 = "Price Qualifier 4"
    PriceQualifier5 = "Price Qualifier 5"
    PriceQualifier6 = "Price Qualifier 6"
    PriceQualifier7 = "Price Qualifier 7"
    PriceQualifier8 = "Price Qualifier 8"

    # Callout Ad Extension
    CalloutAdExtension = "Callout Ad Extension"
    AccountCalloutAdExtension = "Account Callout Ad Extension"
    CampaignCalloutAdExtension = "Campaign Callout Ad Extension"
    AdGroupCalloutAdExtension = "Ad Group Callout Ad Extension"
    CalloutText = "Callout Text"

    # Bid Strategy
    BidStrategyType = "Bid Strategy Type"
    BidStrategyMaxCpc = "Bid Strategy MaxCpc"
    BidStrategyTargetCpa = "Bid Strategy TargetCpa"

    # Remarketing
    RemarketingList = "Remarketing List"
    Audience = "Audience"
    Description = "Description"
    MembershipDuration = "Membership Duration"
    Scope = "Scope"
    TagId = "UET Tag Id"
    AudienceId = "Audience Id"
    RemarketingTargetingSetting = "Remarketing Targeting Setting"
    RemarketingRule = "Remarketing Rule"

    # Expanded Text Ad
    TitlePart1 = "Title Part 1"
    TitlePart2 = "Title Part 2"
    Path1 = "Path 1"
    Path2 = "Path 2"

    # Ad Extension Scheduling
    AdSchedule = "Ad Schedule"
    UseSearcherTimeZone = "Use Searcher Time Zone"

    # Dynamic Search Ads
    DomainLanguage = "Domain Language"
    DynamicAdTargetCondition1 = "Dynamic Ad Target Condition 1"
    DynamicAdTargetValue1 = "Dynamic Ad Target Value 1"
    DynamicAdTargetCondition2 = "Dynamic Ad Target Condition 2"
    DynamicAdTargetValue2 = "Dynamic Ad Target Value 2"
    DynamicAdTargetCondition3 = "Dynamic Ad Target Condition 3"
    DynamicAdTargetValue3 = "Dynamic Ad Target Value 3"
