__author__ = 'Bing Ads SDK Team'
__email__ = 'bing_ads_sdk@microsoft.com'

from .bulk_ad_group_age_criterion import *
from .bulk_ad_group_day_time_criterion import *
from .bulk_ad_group_device_criterion import *
from .bulk_ad_group_gender_criterion import *
from .bulk_ad_group_location_criterion import *
from .bulk_ad_group_location_intent_criterion import *
from .bulk_ad_group_negative_location_criterion import *
from .bulk_ad_group_radius_criterion import *
from .bulk_campaign_age_criterion import *
from .bulk_campaign_day_time_criterion import *
from .bulk_campaign_device_criterion import *
from .bulk_campaign_gender_criterion import *
from .bulk_campaign_location_criterion import *
from .bulk_campaign_location_intent_criterion import *
from .bulk_campaign_negative_location_criterion import *
from .bulk_campaign_radius_criterion import *
